/*
**
**
** MODULE: client.c
**
**
** PROGRAM: Portable SEARCH Application (OpenVMS,DOS,NT,OSF/1,ULTRIX)
**
**
** ABSTRACT: SEARCH is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft platforms.  
**
**          
**
**
*/      
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "search.h"
#ifndef VMS
#include <malloc.h>
#endif

#if defined(MSDOS) || defined(WIN32) 
#include "dceport.h"
#else
#define __RPC_API 
#endif

#ifdef WIN32
#define MAIN_DECL _CRTAPI1 
#else
#define MAIN_DECL
#endif


int 
MAIN_DECL main 
              (
               ac, 
               av
              )
int   ac;
char *av[];
{
    short             search_status;    /* status from search       */
    error_status_t    rpc_status;       /* comm/fault status code   */    
    idl_char          result[LINESIZE]; /* string that matched      */
    idl_char          match[LINESIZE];  /* string to look for       */
    search_spec       custom_handle;    /* search customized handle */
  

    /*
    ** Initialize some strings
    */
    match[0]  = '\0';
    custom_handle.binding[0]  = '\0';
    custom_handle.filename[0] = '\0';

    /*
    ** There should be 4 parameters to searchit:
    **
    **    searchit <hostname> <filename> <matchstring>
    **
    ** where
    **
    **    <hostname> is the hostname where the file to be searched 
    **                 exists.
    **
    **    <filename> is the name of the file to be searched.
    **
    **    <matchstring> is the string to search <filename> for.
    **
    */
    if (ac != 4)
     {
     /* 
     ** Not the right number of parameters
     */ 
     printf("\t\nUsage: searchit <hostname> <filename> <matchstring>\n\n");
     exit(EXIT_FAILURE);
     }

    /*
    ** Set up the string binding, the filename, and the
    **    match string from the command line.
    */
    strcpy ((char *)custom_handle.binding,  "ncacn_ip_tcp:");
    strcat ((char *)custom_handle.binding,  av[1]);
    strcpy ((char *)custom_handle.filename, av[2]);
    strcpy ((char *)match, av[3]);

    /*
    ** Search the given file on the given host for the
    **    given string...
    */
    search_status = searchit 
                            (
                             custom_handle,
                             match,
                             result,
                             &rpc_status
                            );

    /*
    ** Most importantly...did we get an RPC error?
    */
    if (rpc_status)
        {
        printf("RPC error: %d\n",rpc_status);
        exit(EXIT_FAILURE);
        }
 
    /*
    ** Check the return status
    */
    switch (search_status)
        {
        /*
        ** If FILE_ERROR is returned, then we either couldn't find
        **    the requested file, or we couldn't open it.
        */
        case FILE_ERROR: 
            {
            printf("Couldn't find or open requested file\n");
            exit(EXIT_FAILURE);
            }

        /*
        ** If NO_MATCH is returned, then we didn't find the string 
        **    in the file
        */
        case NO_MATCH:
            {
            printf("Couldn't find %s in %s \n",av[3],av[2]);
            exit(EXIT_SUCCESS);
            }

        /*
        ** Otherwise, the returned value is the line number where
        **    we found the match
        */
        default:
            {
            printf("Found a match on line number %d:\n\n\t %s\n",
                   search_status,result);
            break;
            }
        }
 
    return (EXIT_SUCCESS);    
  
}


#if defined(MSDOS) || defined(WIN32)

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );
    
}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}

#endif

/*
** FUNCTION: search_spec_bind 
**
** PURPOSE: search_spec_bind is called from the client
**             stub to establish a customized binding 
**             handle for this remote procedure call.
**
*/
handle_t __RPC_API 
search_spec_bind 
           (
            custom_handle
           )
search_spec custom_handle;
{
    rpc_binding_handle_t xhandle;
    error_status_t       status;

    /*
    ** Display the server in use
    */
    printf("\n\t(Selecting server binding: %s)\n\n", 
           custom_handle.binding);       

    /*
    ** Convert the character string binding into an RPC handle
    */
    rpc_binding_from_string_binding
               (
                custom_handle.binding, 
                &xhandle, 
                &status
               );

    if (status)
        {
        printf("Invalid string binding\n");
        exit (EXIT_FAILURE);
        }
    
    return ( xhandle );

}

/*
**
** FUNCTION: search_spec_unbind
**
** PURPOSE: search_spec_unbind is called by the client stub
**             to free the customized binding handle used
**             by this remote procedure call.
**
*/
void __RPC_API
search_spec_unbind 
           (
            custom_handle,
            xhandle
           )
search_spec custom_handle;
handle_t  xhandle;
{
    error_status_t status;

    /*
    ** Free the binding handle
    */
    rpc_binding_free
               (
                &xhandle,
                &status
               );
    return;
}
