/*
**
**
** MODULE: manager.c
**
**
** PROGRAM: Portable SEARCH Application (OpenVMS,DOS,NT,OSF/1,ULTRIX)
**
**
** ABSTRACT: SEARCH is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft platforms.
**
**
*/      

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef VMS
#include <malloc.h>
#endif
#include "search.h"

#ifdef WIN32 
#include "dceport.h"
#endif

/*
**
** FUNCTION:  getfileline
**
** PURPOSE:
**      Retrieve lines from input file
**
*/
int 
getfileline
           (
            line, 
            file_handle 
           )
idl_char *line;
FILE     *file_handle;
{
    /*
    ** Each call of this routine returns a line of the 
    **    file referred to by the file handle.  On EOF, it 
    **    returns -1.
    */
    char ch;

    while ((ch = fgetc(file_handle)) != '\n' && ch != EOF)
        {
        /*
        ** Tabs are unpredictable, so substitute
        **    three spaces if you run across a tab...
        */
        if (ch == '\t')
            {
            *line++ = ' ';
            *line++ = ' ';
            *line++ = ' ';
            }
        else        
            *line++ = ch;
        
        }

    *line++ = '\0';

    if (ch == EOF)
        return (-1);
    else
        return (0);
}


/*
**
** FUNCTION:  searchit
**
** PURPOSE:   Find a string in a file
**
*/
short 
searchit 
        ( 
         custom_handle,
         stringin, 
         stringout,
         rpc_status 
        )
search_spec       custom_handle;
idl_char          stringin[LINESIZE];
idl_char          stringout[LINESIZE];
error_status_t   *rpc_status;
{
    idl_char buf[LINESIZE];
    FILE *file_handle;
    short line_number = 0;

    *rpc_status = error_status_ok;
    
    /*
    ** Return FILE_ERROR if we can't find the file or can't open it
    */
    if ((file_handle = fopen ((char *)custom_handle.filename,"r")) == NULL)
        return (FILE_ERROR);
               
    /*
    **  Either return the line of the file that contains a string
    **  match, or return NO_MATCH
    */
    while ((getfileline(buf,file_handle)) != -1)
        {
        /*
        ** Keep track of line number
        */
        line_number++;

        /*
        ** Does this line of the file contain the string?
        */
        if ((strstr((char *)buf, (char *)stringin)) != (char *) NULL)
            {
            /*
            ** Yes!  Return the matching line and the line number
            **    after closing the file.
            */
            strcpy ((char *)stringout,(char *)buf);
            fclose(file_handle);
            return(line_number);
            }
        }

    /*
    ** Couldn't find it.  Close the file and return.
    */
    fclose(file_handle);
    return(NO_MATCH);
}
